<?php
session_start();
require_once 'config.php';
require_once 'auth.php';

// Отключаем вывод ошибок
ini_set('display_errors', 0);
error_reporting(0);

// Устанавливаем заголовки для JSON-ответа
header('Content-Type: application/json');

// Определение корневой директории проекта
define('ROOT_PATH', realpath(dirname(__FILE__) . '/../'));

// Функция для логирования ошибок
function logError($message) {
    error_log(date('Y-m-d H:i:s') . ' - ' . $message . "\n", 3, ROOT_PATH . '/logs/track_actions_errors.log');
}

// Функция для отправки JSON-ответа
function sendJsonResponse($data) {
    echo json_encode($data);
    exit;
}

// Проверка авторизации
if (!isset($_SESSION['user_id'])) {
    sendJsonResponse(['status' => 'error', 'message' => 'Пользователь не авторизован']);
}


// Добавим новую функцию для отправки трека на модерацию
function sendTrackToModeration($trackId) {
    global $mysqli;
    $user_id = $_SESSION['user_id'];

    $stmt = $mysqli->prepare("UPDATE tracks SET status = 'moderation' WHERE id = ? AND user_id = ? AND status = 'draft'");
    $stmt->bind_param("ii", $trackId, $user_id);

    if ($stmt->execute()) {
        return ['status' => 'success', 'message' => 'Трек отправлен на модерацию'];
    } else {
        return ['status' => 'error', 'message' => 'Ошибка при отправке трека на модерацию'];
    }
}

function getTrack($trackId) {
    global $mysqli;
    $user_id = $_SESSION['user_id'];

    $stmt = $mysqli->prepare("SELECT * FROM tracks WHERE id = ? AND user_id = ?");
    $stmt->bind_param("ii", $trackId, $user_id);
    $stmt->execute();
    $result = $stmt->get_result();

    if ($result->num_rows === 0) {
        return ['status' => 'error', 'message' => 'Трек не найден'];
    }

    $track = $result->fetch_assoc();
    return ['status' => 'success', 'track' => $track];
}

function updateTrack($trackId, $data) {
    global $mysqli;
    $user_id = $_SESSION['user_id'];

    // Проверяем статус трека перед обновлением
    $stmt = $mysqli->prepare("SELECT status FROM tracks WHERE id = ? AND user_id = ?");
    $stmt->bind_param("ii", $trackId, $user_id);
    $stmt->execute();
    $result = $stmt->get_result();
    $track = $result->fetch_assoc();

    if ($track['status'] == 'moderation' || $track['status'] == 'approved') {
        return ['status' => 'error', 'message' => 'Нельзя редактировать трек, который находится на модерации или уже одобрен'];
    }

    $stmt = $mysqli->prepare("UPDATE tracks SET artist_name = ?, track_name = ?, lyrics = ?, music_author = ?, lyrics_author = ? WHERE id = ? AND user_id = ?");
    $stmt->bind_param("sssssii", $data['artistName'], $data['trackName'], $data['lyrics'], $data['musicAuthor'], $data['lyricsAuthor'], $trackId, $user_id);

    if ($stmt->execute()) {
        return ['status' => 'success', 'message' => 'Трек успешно обновлен'];
    } else {
        return ['status' => 'error', 'message' => 'Ошибка при обновлении трека'];
    }
}

function deleteTrack($trackId) {
    global $mysqli;
    $user_id = $_SESSION['user_id'];

    // Проверяем статус трека перед удалением
    $stmt = $mysqli->prepare("SELECT status, file_path, cover_path FROM tracks WHERE id = ? AND user_id = ?");
    $stmt->bind_param("ii", $trackId, $user_id);
    $stmt->execute();
    $result = $stmt->get_result();
    $track = $result->fetch_assoc();

    if ($track['status'] == 'moderation' || $track['status'] == 'approved') {
        return ['status' => 'error', 'message' => 'Нельзя удалить трек, который находится на модерации или уже одобрен'];
    }

    // Удаляем файлы трека и обложки
    if (file_exists($track['file_path'])) {
        unlink($track['file_path']);
    }
    if (file_exists($track['cover_path'])) {
        unlink($track['cover_path']);
    }

    // Удаляем запись из базы данных
    $stmt = $mysqli->prepare("DELETE FROM tracks WHERE id = ? AND user_id = ?");
    $stmt->bind_param("ii", $trackId, $user_id);

    if ($stmt->execute()) {
        return ['status' => 'success', 'message' => 'Трек успешно удален'];
    } else {
        return ['status' => 'error', 'message' => 'Ошибка при удалении трека'];
    }
}

function getTracks() {
    global $mysqli;
    $user_id = $_SESSION['user_id'];

    $stmt = $mysqli->prepare("SELECT * FROM tracks WHERE user_id = ? ORDER BY created_at DESC");
    $stmt->bind_param("i", $user_id);
    $stmt->execute();
    $result = $stmt->get_result();

    $tracks = [];
    while ($row = $result->fetch_assoc()) {
        $tracks[] = $row;
    }

    return ['status' => 'success', 'tracks' => $tracks];
}

function uploadTrack() {
    global $mysqli;
    $user_id = $_SESSION['user_id'];

    logError("Начало загрузки трека для пользователя ID: $user_id");

    // Проверка и обработка входных данных
    $artist_name = filter_input(INPUT_POST, 'artistName', FILTER_SANITIZE_STRING);
    $track_name = filter_input(INPUT_POST, 'trackName', FILTER_SANITIZE_STRING);
    $lyrics = filter_input(INPUT_POST, 'lyrics', FILTER_SANITIZE_STRING);
    $music_author = filter_input(INPUT_POST, 'musicAuthor', FILTER_SANITIZE_STRING);
    $lyrics_author = filter_input(INPUT_POST, 'lyricsAuthor', FILTER_SANITIZE_STRING);

    // Проверка обязательных полей
    if (empty($artist_name) || empty($track_name) || empty($music_author) || empty($lyrics_author)) {
        return ['status' => 'error', 'message' => 'Пожалуйста, заполните все обязательные поля'];
    }

    logError("Данные формы получены: artist_name=$artist_name, track_name=$track_name");

    // Проверка загруженного файла трека
    if (!isset($_FILES['trackFile']) || $_FILES['trackFile']['error'] !== UPLOAD_ERR_OK) {
        $error_message = isset($_FILES['trackFile']) ? getFileUploadErrorMessage($_FILES['trackFile']['error']) : 'Файл трека не был загружен';
        logError("Ошибка при загрузке файла трека: " . $error_message);
        return ['status' => 'error', 'message' => $error_message];
    }

    $track_file = $_FILES['trackFile'];
    $file_extension = strtolower(pathinfo($track_file['name'], PATHINFO_EXTENSION));

    logError("Файл трека получен: name=" . $track_file['name'] . ", size=" . $track_file['size'] . ", type=" . $track_file['type']);

    // Проверка типа файла
    if (!in_array($file_extension, ['wav', 'mp3'])) {
        logError("Недопустимый формат файла трека: $file_extension");
        return ['status' => 'error', 'message' => 'Недопустимый формат файла. Разрешены только WAV и MP3'];
    }

    // Проверка размера файла
    if ($track_file['size'] > 100 * 1024 * 1024) {
        logError("Размер файла трека превышает лимит: " . $track_file['size'] . " байт");
        return ['status' => 'error', 'message' => 'Размер файла превышает 100 МБ'];
    }

    // Проверка загруженной обложки
    if (!isset($_FILES['coverImage']) || $_FILES['coverImage']['error'] !== UPLOAD_ERR_OK) {
        $error_message = isset($_FILES['coverImage']) ? getFileUploadErrorMessage($_FILES['coverImage']['error']) : 'Файл обложки не был загружен';
        logError("Ошибка при загрузке обложки: " . $error_message);
        return ['status' => 'error', 'message' => $error_message];
    }

    $cover_file = $_FILES['coverImage'];
    $cover_extension = strtolower(pathinfo($cover_file['name'], PATHINFO_EXTENSION));

    logError("Файл обложки получен: name=" . $cover_file['name'] . ", size=" . $cover_file['size'] . ", type=" . $cover_file['type']);

    // Проверка типа файла обложки
    if (!in_array($cover_extension, ['jpg', 'jpeg', 'png', 'gif'])) {
        logError("Недопустимый формат файла обложки: $cover_extension");
        return ['status' => 'error', 'message' => 'Недопустимый формат файла обложки. Разрешены только JPG, JPEG, PNG и GIF'];
    }

    // Генерация уникальных имен файлов
    $track_filename = uniqid() . '.' . $file_extension;
    $cover_filename = uniqid() . '.' . $cover_extension;

    // Пути для сохранения файлов
    $tracks_dir = ROOT_PATH . '/uploads/tracks/';
    $covers_dir = ROOT_PATH . '/uploads/covers/';
    $track_path = $tracks_dir . $track_filename;
    $cover_path = $covers_dir . $cover_filename;

    logError("Пути для сохранения файлов: track_path=$track_path, cover_path=$cover_path");

    // Проверка существования директорий и прав доступа
    if (!is_dir($tracks_dir) || !is_writable($tracks_dir)) {
        logError("Директория для треков не существует или недоступна для записи: $tracks_dir");
        return ['status' => 'error', 'message' => 'Ошибка сервера при сохранении трека'];
    }
    if (!is_dir($covers_dir) || !is_writable($covers_dir)) {
        logError("Директория для обложек не существует или недоступна для записи: $covers_dir");
        return ['status' => 'error', 'message' => 'Ошибка сервера при сохранении обложки'];
    }

    // Перемещение загруженных файлов
    if (!move_uploaded_file($track_file['tmp_name'], $track_path)) {
        logError("Ошибка при перемещении файла трека: " . error_get_last()['message']);
        return ['status' => 'error', 'message' => 'Ошибка при сохранении файла трека'];
    }
    if (!move_uploaded_file($cover_file['tmp_name'], $cover_path)) {
        logError("Ошибка при перемещении файла обложки: " . error_get_last()['message']);
        unlink($track_path); // Удаляем трек, если не удалось сохранить обложку
        return ['status' => 'error', 'message' => 'Ошибка при сохранении файла обложки'];
    }

    logError("Файлы успешно перемещены");

    // Измените запрос на вставку, добавив столбец status
    $stmt = $mysqli->prepare("INSERT INTO tracks (user_id, artist_name, track_name, file_path, cover_path, lyrics, music_author, lyrics_author, status) VALUES (?, ?, ?, ?, ?, ?, ?, ?, 'draft')");
    $stmt->bind_param("isssssss", $user_id, $artist_name, $track_name, $track_path, $cover_path, $lyrics, $music_author, $lyrics_author);

    if ($stmt->execute()) {
        logError("Информация о треке успешно сохранена в базе данных");
        return ['status' => 'success', 'message' => 'Трек успешно загружен'];
    } else {
        logError("Ошибка при сохранении информации о треке в базе данных: " . $mysqli->error);
        unlink($track_path);
        unlink($cover_path);
        return ['status' => 'error', 'message' => 'Ошибка при сохранении информации о треке в базе данных'];
    }
}

// Вспомогательная функция для получения сообщений об ошибках загрузки файлов
function getFileUploadErrorMessage($error_code) {
    switch ($error_code) {
        case UPLOAD_ERR_INI_SIZE:
            return "Размер файла превышает максимально допустимый размер, указанный в php.ini";
        case UPLOAD_ERR_FORM_SIZE:
            return "Размер файла превышает максимально допустимый размер, указанный в HTML-форме";
        case UPLOAD_ERR_PARTIAL:
            return "Файл был загружен только частично";
        case UPLOAD_ERR_NO_FILE:
            return "Файл не был загружен";
        case UPLOAD_ERR_NO_TMP_DIR:
            return "Отсутствует временная папка для загрузки файлов";
        case UPLOAD_ERR_CANT_WRITE:
            return "Не удалось записать файл на диск";
        case UPLOAD_ERR_EXTENSION:
            return "Загрузка файла была остановлена расширением PHP";
        default:
            return "Неизвестная ошибка при загрузке файла";
    }
}

// Обработка запроса
if (isset($_GET['action'])) {
    switch ($_GET['action']) {
      case 'send_to_moderation':
        $trackId = $_POST['trackId'];
        $result = sendTrackToModeration($trackId);
        sendJsonResponse($result);
        break;

        case 'approve_track':
                $trackId = $_POST['trackId'];
                $result = approveTrack($trackId);
                echo json_encode(['status' => $result ? 'success' : 'error']);
                break;
            case 'reject_track':
                $trackId = $_POST['trackId'];
                $comment = $_POST['comment'];
                $result = rejectTrack($trackId, $comment);
                echo json_encode(['status' => $result ? 'success' : 'error']);
                break;
            case 'request_changes':
                $trackId = $_POST['trackId'];
                $comment = $_POST['comment'];
                $result = requestChanges($trackId, $comment);
                echo json_encode(['status' => $result ? 'success' : 'error']);
                break;

        case 'upload':
            $result = uploadTrack();
            sendJsonResponse($result);
            break;
        case 'get_track':
            $trackId = $_GET['trackId'];
            $result = getTrack($trackId);
            sendJsonResponse($result);
            break;
        case 'update_track':
            $trackId = $_POST['trackId'];
            $result = updateTrack($trackId, $_POST);
            sendJsonResponse($result);
            break;
        case 'delete_track':
            $trackId = $_POST['trackId'];
            $result = deleteTrack($trackId);
            sendJsonResponse($result);
            break;
        case 'get_tracks':
            $result = getTracks();
            sendJsonResponse($result);
            break;
        default:
            sendJsonResponse(['status' => 'error', 'message' => 'Неизвестное действие']);
    }
} else {
    sendJsonResponse(['status' => 'error', 'message' => 'Действие не указано']);
}
